"use strict";
var LoggerOptions_1 = require("../log/LoggerOptions");
var LoggerFactoryService_1 = require("../log/standard/LoggerFactoryService");
var DataStructures_1 = require("../utils/DataStructures");
var LoggerControlImpl = (function () {
    function LoggerControlImpl() {
    }
    /*
     setLogLevel(level: string, idGroup: number | null = null)
     ** Set new log level for id of LogGroup, applied to both new and existing loggers. If id is null applies to all groups.
     ** level must be one of: Trace,Debug,Info,Warn,Error,Fatal.
    
     setLogFormat(format: string, showTimestamp: boolean = true, showLoggerName: boolean = true, idGroup: number | null = null)
     ** Set format for logging, format must be one of: "Default, YearMonthDayTime, YearDayMonthWithFullTime, YearDayMonthTime".
     ** Applied to both new and existing loggers.
     ** If id is null applies to all groups.
    
     reset(id: number | null = null)
     ** Reset LogGroup back to original settings, applied to both new and existing loggers.
     ** If id is null applies to all groups.
       */
    LoggerControlImpl.prototype.help = function () {
        /* tslint:disable:no-console */
        console.log(LoggerControlImpl._help);
        /* tslint:enable:no-console */
    };
    LoggerControlImpl.prototype.listFactories = function () {
        var rtSettingsFactories = LoggerControlImpl._getRuntimeSettingsLoggerFactories();
        var result = new DataStructures_1.StringBuilder();
        result.appendLine("Registered LoggerFactories (index / name)");
        for (var i = 0; i < rtSettingsFactories.length; i++) {
            var rtSettingsFactory = rtSettingsFactories[i];
            result.append("  " + i).append(": " + rtSettingsFactory.getName() + "\n");
        }
        /* tslint:disable:no-console */
        console.log(result.toString());
        /* tslint:enable:no-console */
    };
    LoggerControlImpl.prototype.showSettings = function (id) {
        if (id === void 0) { id = null; }
        var result = [];
        if (id == null) {
            var idx_1 = 0;
            LoggerControlImpl._getSettings().getRuntimeSettingsForLoggerFactories().forEach(function (item) {
                result.push(new DataStructures_1.TuplePair(idx_1++, item));
            });
        }
        else {
            var settings = LoggerControlImpl._getRuntimeSettingsLoggerFactories();
            if (id >= 0 && id < settings.length) {
                result.push(new DataStructures_1.TuplePair(id, settings[id]));
            }
            else {
                throw new Error("Requested number: " + id + " was not found.");
            }
        }
        for (var _i = 0, result_1 = result; _i < result_1.length; _i++) {
            var setting = result_1[_i];
            /* tslint:disable:no-console */
            console.log("  LoggerFactory: " + setting.y.getName() + " (id=" + setting.x + ")");
            var logGroupRuntimeSettings = setting.y.getLogGroupRuntimeSettings();
            for (var g = 0; g < logGroupRuntimeSettings.length; g++) {
                var groupSetting = logGroupRuntimeSettings[g];
                console.log("     LogGroup: (id=" + g + ")");
                console.log("       RegExp: " + groupSetting.logGroupRule.regExp.source);
                console.log("       Level: " + LoggerOptions_1.LogLevel[groupSetting.level].toString());
                console.log("       LoggerType: " + LoggerOptions_1.LoggerType[groupSetting.loggerType].toString());
            }
        }
    };
    LoggerControlImpl.prototype.getLoggerFactoryControl = function (idFactory) {
        if (idFactory === null) {
            throw new Error("idFactory argument is required");
        }
        var loggerFactoriesSettings = LoggerControlImpl._getSettings().getRuntimeSettingsForLoggerFactories();
        if (idFactory >= 0 && idFactory < loggerFactoriesSettings.length) {
            return new LoggerFactoryControlImpl(loggerFactoriesSettings[idFactory]);
        }
        throw new Error("idFactory is invalid (less than 0) or non existing id.");
    };
    LoggerControlImpl._getRuntimeSettingsLoggerFactories = function () {
        return LoggerControlImpl._getSettings().getRuntimeSettingsForLoggerFactories();
    };
    LoggerControlImpl._getSettings = function () {
        return LoggerFactoryService_1.LFService.getRuntimeSettings();
    };
    return LoggerControlImpl;
}());
LoggerControlImpl._help = "\n  help():\n    ** Shows this help.\n    \n  listFactories()\n    ** List all registered LoggerFactories with associated log groups with respective ids (ids can be used to target a factory and/or group).\n    \n  showSettings(idFactory: number | null)\n    ** Show log group settings for idFactory (use list() to find id for a LoggerFactory). If idFactory is null applies to all factories. \n  \n  getLoggerFactoryControl(idFactory: number): LoggerFactoryControl\n    ** Return LoggerFactoryControl when found for given idFactory or throws Error if invalid or null, get the id by using listFactories()\n";
exports.LoggerControlImpl = LoggerControlImpl;
var LoggerFactoryControlImpl = (function () {
    function LoggerFactoryControlImpl(settings) {
        this._settings = settings;
    }
    LoggerFactoryControlImpl.prototype.help = function () {
        /* tslint:disable:no-console */
        console.log(LoggerFactoryControlImpl._help);
        /* tslint:enable:no-console */
    };
    LoggerFactoryControlImpl.prototype.listLogGroups = function () {
        var result = new DataStructures_1.StringBuilder();
        var logGroupRuntimeSettings = this._settings.getLogGroupRuntimeSettings();
        result.appendLine("Registered LogGroups (index / expression)");
        for (var i = 0; i < logGroupRuntimeSettings.length; i++) {
            var logGroupRuntimeSetting = logGroupRuntimeSettings[i];
            result.appendLine("  " + i + ": " + logGroupRuntimeSetting.logGroupRule.regExp.source + ", logLevel=" +
                LoggerOptions_1.LogLevel[logGroupRuntimeSetting.level].toString() + ", showTimestamp=" + logGroupRuntimeSetting.logFormat.showTimeStamp +
                ", showLoggerName=" + logGroupRuntimeSetting.logFormat.showLoggerName +
                ", format=" + LoggerOptions_1.DateFormatEnum[logGroupRuntimeSetting.logFormat.dateFormat.formatEnum].toString());
        }
        /* tslint:disable:no-console */
        console.log(result.toString());
        /* tslint:enable:no-console */
    };
    LoggerFactoryControlImpl.prototype.setLogLevel = function (level, idGroup) {
        if (idGroup === void 0) { idGroup = null; }
        var newLevel = LoggerOptions_1.LogLevel.fromString(level);
        var settings = this._getLogGroupRunTimeSettingsFor(idGroup);
        for (var _i = 0, settings_1 = settings; _i < settings_1.length; _i++) {
            var setting = settings_1[_i];
            setting.level = newLevel;
        }
        /* tslint:disable:no-console */
        console.log("LogLevel set to " + level + " for " + (idGroup != null ? " LogGroup " + idGroup + "." : " all LogGroups."));
        /* tslint:enable:no-console */
    };
    LoggerFactoryControlImpl.prototype.setLogFormat = function (format, showTimestamp, showLoggerName, idGroup) {
        if (showTimestamp === void 0) { showTimestamp = true; }
        if (showLoggerName === void 0) { showLoggerName = true; }
        if (idGroup === void 0) { idGroup = null; }
        var formatEnum = LoggerOptions_1.DateFormatEnum.fromString(format);
        var settings = this._getLogGroupRunTimeSettingsFor(idGroup);
        for (var _i = 0, settings_2 = settings; _i < settings_2.length; _i++) {
            var setting = settings_2[_i];
            setting.logFormat.dateFormat.formatEnum = formatEnum;
            setting.logFormat.showTimeStamp = showTimestamp;
            setting.logFormat.showLoggerName = showLoggerName;
        }
        /* tslint:disable:no-console */
        console.log("LogFormat set to " + format + " for " + (idGroup != null ? " LogGroup " + idGroup + "." : " all LogGroups."));
        /* tslint:enable:no-console */
    };
    LoggerFactoryControlImpl.prototype.reset = function (idGroup) {
        if (idGroup === void 0) { idGroup = null; }
        var settings = this._getLogGroupRunTimeSettingsFor(idGroup);
        for (var _i = 0, settings_3 = settings; _i < settings_3.length; _i++) {
            var setting = settings_3[_i];
            setting.level = setting.logGroupRule.level;
            setting.logFormat.showTimeStamp = setting.logGroupRule.logFormat.showTimeStamp;
            setting.logFormat.showLoggerName = setting.logGroupRule.logFormat.showLoggerName;
            setting.logFormat.dateFormat.formatEnum = setting.logGroupRule.logFormat.dateFormat.formatEnum;
        }
        /* tslint:disable:no-console */
        console.log("Reset all settings for " + (idGroup != null ? " LogGroup " + idGroup + "." : " all LogGroups."));
        /* tslint:enable:no-console */
    };
    LoggerFactoryControlImpl.prototype._getLogGroupRunTimeSettingsFor = function (idGroup) {
        var settings = [];
        if (idGroup !== null) {
            this._checkIndex(idGroup);
            settings.push(this._settings.getLogGroupRuntimeSettings()[idGroup]);
        }
        else {
            settings = this._settings.getLogGroupRuntimeSettings();
        }
        return settings;
    };
    LoggerFactoryControlImpl.prototype._checkIndex = function (index) {
        if (index < 0 || index >= this._settings.getLogGroupRuntimeSettings().length) {
            throw new Error("Invalid index, use listLogGroups to find out a valid one.");
        }
    };
    return LoggerFactoryControlImpl;
}());
LoggerFactoryControlImpl._help = "\n  help():\n    ** Shows this help.\n    \n  listLogGroups()\n    ** Lists all registered LogGroups for this factory and their current settings.\n    \n  setLogLevel(level: string, idGroup: number | null = null)\n    ** Sets the log level for given LogGroup id, or if null for all LogGroups.\n     \n  setLogFormat(format: string, showTimestamp: boolean = true, showLoggerName: boolean = true, idGroup: number | null = null)\n    ** Sets the log format, whether to show a timestamp and whether to show a logger name for given group or all groups if null.\n  \n  reset(idGroup: number | null = null)\n    ** Reset everything back to original defaults for given LogGroup id, or all if null.\n";
//# sourceMappingURL=LogGroupControl.js.map