"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var DataStructures_1 = require("../../utils/DataStructures");
var MessageUtils_1 = require("../../utils/MessageUtils");
var LoggerOptions_1 = require("../LoggerOptions");
var Message = (function () {
    function Message(ready, logLevel, message) {
        if (message === void 0) { message = null; }
        this._ready = ready;
        this._logLevel = logLevel;
        this._message = message;
    }
    Object.defineProperty(Message.prototype, "ready", {
        get: function () {
            return this._ready;
        },
        set: function (value) {
            this._ready = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Message.prototype, "message", {
        get: function () {
            return this._message;
        },
        set: function (value) {
            this._message = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Message.prototype, "logLevel", {
        get: function () {
            return this._logLevel;
        },
        enumerable: true,
        configurable: true
    });
    return Message;
}());
exports.Message = Message;
/**
 * Abstract base logger, extend to easily implement a custom logger that
 * logs wherever you want. You only need to implement doLog(msg: string) and
 * log that somewhere (it will contain format and everything else).
 */
var AbstractLogger = (function () {
    function AbstractLogger(name, rule) {
        this._allMessages = new DataStructures_1.LinkedList();
        this._open = true;
        this._name = name;
        this._rule = rule;
        this._level = rule.level;
    }
    Object.defineProperty(AbstractLogger.prototype, "name", {
        get: function () {
            return this._name;
        },
        enumerable: true,
        configurable: true
    });
    AbstractLogger.prototype.trace = function (msg, error) {
        if (error === void 0) { error = null; }
        this._log(LoggerOptions_1.LogLevel.Trace, msg, error);
    };
    AbstractLogger.prototype.debug = function (msg, error) {
        if (error === void 0) { error = null; }
        this._log(LoggerOptions_1.LogLevel.Debug, msg, error);
    };
    AbstractLogger.prototype.info = function (msg, error) {
        if (error === void 0) { error = null; }
        this._log(LoggerOptions_1.LogLevel.Info, msg, error);
    };
    AbstractLogger.prototype.warn = function (msg, error) {
        if (error === void 0) { error = null; }
        this._log(LoggerOptions_1.LogLevel.Warn, msg, error);
    };
    AbstractLogger.prototype.error = function (msg, error) {
        if (error === void 0) { error = null; }
        this._log(LoggerOptions_1.LogLevel.Error, msg, error);
    };
    AbstractLogger.prototype.fatal = function (msg, error) {
        if (error === void 0) { error = null; }
        this._log(LoggerOptions_1.LogLevel.Fatal, msg, error);
    };
    AbstractLogger.prototype.tracec = function (msg, error) {
        this._logc(LoggerOptions_1.LogLevel.Trace, msg, error);
    };
    AbstractLogger.prototype.debugc = function (msg, error) {
        this._logc(LoggerOptions_1.LogLevel.Debug, msg, error);
    };
    AbstractLogger.prototype.infoc = function (msg, error) {
        this._logc(LoggerOptions_1.LogLevel.Info, msg, error);
    };
    AbstractLogger.prototype.warnc = function (msg, error) {
        this._logc(LoggerOptions_1.LogLevel.Warn, msg, error);
    };
    AbstractLogger.prototype.errorc = function (msg, error) {
        this._logc(LoggerOptions_1.LogLevel.Error, msg, error);
    };
    AbstractLogger.prototype.fatalc = function (msg, error) {
        this._logc(LoggerOptions_1.LogLevel.Fatal, msg, error);
    };
    AbstractLogger.prototype.isTraceEnabled = function () {
        return this._level === LoggerOptions_1.LogLevel.Trace;
    };
    AbstractLogger.prototype.isDebugEnabled = function () {
        return this._level <= LoggerOptions_1.LogLevel.Debug;
    };
    AbstractLogger.prototype.isInfoEnabled = function () {
        return this._level <= LoggerOptions_1.LogLevel.Info;
    };
    AbstractLogger.prototype.isWarnEnabled = function () {
        return this._level <= LoggerOptions_1.LogLevel.Warn;
    };
    AbstractLogger.prototype.isErrorEnabled = function () {
        return this._level <= LoggerOptions_1.LogLevel.Error;
    };
    AbstractLogger.prototype.isFatalEnabled = function () {
        return this._level <= LoggerOptions_1.LogLevel.Fatal;
    };
    AbstractLogger.prototype.getLogLevel = function () {
        return this._level;
    };
    AbstractLogger.prototype.isOpen = function () {
        return this._open;
    };
    AbstractLogger.prototype.close = function () {
        this._open = false;
        this._allMessages.clear();
    };
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    AbstractLogger.prototype._addMessage = function (msg) {
        // Make sure to make Message internal class again when this one is gone!
        this._allMessages.addTail(msg);
    };
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    AbstractLogger.prototype._log = function (level, msg, error) {
        if (error === void 0) { error = null; }
        if (this._open && this._level <= level) {
            this._allMessages.addTail(this.createMessage(level, msg, new Date(), error));
            this.processMessages();
        }
    };
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    AbstractLogger.prototype._logc = function (level, msg, error) {
        if (this._open && this._level <= level) {
            this._allMessages.addTail(this.createMessage(level, msg(), new Date(), error !== undefined && error != null ? error() : null));
            this.processMessages();
        }
    };
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    AbstractLogger.prototype.createMessage = function (level, msg, date, error) {
        var _this = this;
        if (error === void 0) { error = null; }
        var result = "";
        if (this._rule.logFormat.showTimeStamp) {
            result += MessageUtils_1.MessageFormatUtils.renderDate(date, this._rule.logFormat.dateFormat) + " ";
        }
        result += LoggerOptions_1.LogLevel[level].toUpperCase() + " ";
        if (this._rule.logFormat.showLoggerName) {
            result += "[" + this._name + "]";
        }
        result += " " + msg;
        if (error != null) {
            var message_1 = new Message(false, level);
            MessageUtils_1.MessageFormatUtils.renderError(error).then(function (stackResult) {
                result += "\n" + stackResult;
                message_1.message = result;
                message_1.ready = true;
                _this.processMessages();
            });
            return message_1;
        }
        return new Message(true, level, result);
    };
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    AbstractLogger.prototype.processMessages = function () {
        // Basically we wait until errors are resolved (those messages
        // may not be ready).
        var msgs = this._allMessages;
        if (msgs.getSize() > 0) {
            do {
                var msg = msgs.getHead();
                if (msg != null) {
                    if (!msg.ready) {
                        break;
                    }
                    msgs.removeHead();
                    // This can never be null normally, but strict null checking ...
                    if (msg.message != null) {
                        this.doLog(msg.message, msg.logLevel);
                    }
                }
            } while (msgs.getSize() > 0);
        }
    };
    return AbstractLogger;
}());
exports.AbstractLogger = AbstractLogger;
/**
 * Future base logger, temporal name will replace AbstractLogger in future 0.3 release.
 */
var AbstractBaseLogger = (function (_super) {
    __extends(AbstractBaseLogger, _super);
    function AbstractBaseLogger(name, logGroupRuntimeSettings) {
        var _this = _super.call(this, name, logGroupRuntimeSettings.logGroupRule) || this;
        _this._logGroupRuntimeSettings = logGroupRuntimeSettings;
        return _this;
    }
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    AbstractBaseLogger.prototype._log = function (level, msg, error) {
        if (error === void 0) { error = null; }
        if (this._open && this._logGroupRuntimeSettings.level <= level) {
            this._addMessage(this.createMessage(level, msg, new Date(), error));
            this.processMessages();
        }
    };
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    AbstractBaseLogger.prototype._logc = function (level, msg, error) {
        if (this._open && this._logGroupRuntimeSettings.level <= level) {
            this._addMessage(this.createMessage(level, msg(), new Date(), error !== undefined && error != null ? error() : null));
            this.processMessages();
        }
    };
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    AbstractBaseLogger.prototype.createMessage = function (level, msg, date, error) {
        var _this = this;
        if (error === void 0) { error = null; }
        var result = "";
        if (this._logGroupRuntimeSettings.logFormat.showTimeStamp) {
            result += MessageUtils_1.MessageFormatUtils.renderDate(date, this._logGroupRuntimeSettings.logFormat.dateFormat) + " ";
        }
        result += LoggerOptions_1.LogLevel[level].toUpperCase() + " ";
        if (this._logGroupRuntimeSettings.logFormat.showLoggerName) {
            result += "[" + this._name + "]";
        }
        result += " " + msg;
        if (error != null) {
            var message_2 = new Message(false, level);
            MessageUtils_1.MessageFormatUtils.renderError(error).then(function (stackResult) {
                result += "\n" + stackResult;
                message_2.message = result;
                message_2.ready = true;
                _this.processMessages();
            });
            return message_2;
        }
        return new Message(true, level, result);
    };
    return AbstractBaseLogger;
}(AbstractLogger));
exports.AbstractBaseLogger = AbstractBaseLogger;
/**
 * Simple logger, that logs to the console. If the console is unavailable will throw exception.
 */
var ConsoleLoggerImpl = (function (_super) {
    __extends(ConsoleLoggerImpl, _super);
    function ConsoleLoggerImpl(name, logGroupRuntimeSettings) {
        return _super.call(this, name, logGroupRuntimeSettings) || this;
    }
    ConsoleLoggerImpl.prototype.doLog = function (msg, logLevel) {
        if (console !== undefined) {
            var logged = false;
            /* tslint:disable:no-console */
            switch (logLevel) {
                case LoggerOptions_1.LogLevel.Trace:
                    // Do not try trace we don't want a stack
                    break;
                case LoggerOptions_1.LogLevel.Debug:
                    if (console.debug) {
                        console.debug(msg);
                        logged = true;
                    }
                    break;
                case LoggerOptions_1.LogLevel.Info:
                    if (console.info) {
                        console.info(msg);
                        logged = true;
                    }
                    break;
                case LoggerOptions_1.LogLevel.Warn:
                    if (console.warn) {
                        console.warn(msg);
                        logged = true;
                    }
                    break;
                case LoggerOptions_1.LogLevel.Error:
                case LoggerOptions_1.LogLevel.Fatal:
                    if (console.error) {
                        console.error(msg);
                        logged = true;
                    }
                    break;
                default:
                    throw new Error("Log level not supported: " + logLevel);
            }
            if (!logged) {
                console.log(msg);
            }
        }
        else {
            throw new Error("Console is not defined, cannot log msg: " + msg);
        }
    };
    return ConsoleLoggerImpl;
}(AbstractBaseLogger));
exports.ConsoleLoggerImpl = ConsoleLoggerImpl;
/**
 * Logger which buffers all messages, use with care due to possible high memory footprint.
 * Can be convenient in some cases. Call toString() for full output, or cast to this class
 * and call getMessages() to do something with it yourself.
 */
var MessageBufferLoggerImpl = (function (_super) {
    __extends(MessageBufferLoggerImpl, _super);
    function MessageBufferLoggerImpl(name, logGroupRuntimeSettings) {
        var _this = _super.call(this, name, logGroupRuntimeSettings) || this;
        _this.messages = [];
        return _this;
    }
    MessageBufferLoggerImpl.prototype.close = function () {
        this.messages = [];
        _super.prototype.close.call(this);
    };
    MessageBufferLoggerImpl.prototype.getMessages = function () {
        return this.messages;
    };
    MessageBufferLoggerImpl.prototype.toString = function () {
        return this.messages.map(function (msg) {
            return msg;
        }).join("\n");
    };
    MessageBufferLoggerImpl.prototype.doLog = function (msg, logLevel) {
        this.messages.push(msg);
    };
    return MessageBufferLoggerImpl;
}(AbstractBaseLogger));
exports.MessageBufferLoggerImpl = MessageBufferLoggerImpl;
//# sourceMappingURL=LoggerImpl.js.map